PROGRAM test_function_pointers
!
!  Purpose:
!    To test Fortran procedure pointers.  The function
!    point will work with any procedure with an explicit
!    interface that has same signature as "func1".
!
!  Record of revisions:
!      Date       Programmer          Description of change
!      ====       ==========          =====================
!    01/08/16    S. J. Chapman        Original code
!
USE test_functions
IMPLICIT NONE

! Declare variables
INTEGER :: index                    ! Selection index
PROCEDURE(func1), POINTER :: p      ! Function pointer
REAL :: x                           ! Calling argument

! Get the name of the file containing the input data.
WRITE (*,*) 'Select a function to associate with the pointer:'
WRITE (*,*) '  1: func1'
WRITE (*,*) '  2: func2'
WRITE (*,*) '  3: func3'
READ (*,*) index

! Is it valid?
IF ( (index < 1) .OR. (index > 3) ) THEN
   
   WRITE (*,*) 'Invalid selection made!'
   ERROR STOP 'Bad index'
   
ELSE

   ! Associate the pointer
   SELECT CASE (index)
      CASE (1)
         WRITE (*,*) 'func1 selected...'
         p => func1
      CASE (2)
         WRITE (*,*) 'func2 selected...'
         p => func2
      CASE (3)
         WRITE (*,*) 'func3 selected...'
         p => func3
   END SELECT
   
   ! Execute the function
   WRITE (*,'(A)',ADVANCE='NO') 'Enter x: ' 
   READ (*,*) x
   WRITE (*,'(A,F13.6)') 'f(x) = ', p(x)
   
END IF

END PROGRAM test_function_pointers
